{-------------------------------------------------------------------------------
Delphi Save And Restore
Copyright (C) 2009 ouiouioui contact@toutenvrac.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-------------------------------------------------------------------------------}
Unit MainFunctions;

Interface

Uses
  StdCtrls, FileCtrl, Forms, Windows, SysUtils, Registry, WinSkinData,
  WinSkinStore, MainMsg, ZipMstr, DateUtils, Utils, IniFiles, Classes,
  ThreadUpdater, ShellApi;

Type
  TSetlStatus          = Procedure(Msg: String);
  TSetpbStatusPosition = Procedure(Position: Integer; Max: Integer = 0);

Function MFSelectDirectory(Describe: String; MyEdit: TEdit; Options: TSelectDirExtOpts = [sdNewUI]): Boolean;
Procedure MFDelEmptyDir(MyListBox: TListBox);
Procedure MFDelEmptyKey(MyListBox: TListBox);
Procedure MFDelSelectedItem(MyListBox: TListBox);
Procedure MFAddDirInListBox(MyEdit: TEdit; MyListBox: TListBox);
Procedure MFReplaceDirInListBox(MyEdit: TEdit; MyListBox: TListBox);
Procedure MFAddKeyInListBox(MyEdit: TEdit; MyListBox: TListBox);
Procedure MFReplaceKeyInListBox(MyEdit: TEdit; MyListBox: TListBox);
Procedure MFSetSkinSelected(Var MySkinData: TSkinData; MyItemIndex: Integer);
Procedure MFStartSave(NameForSave, DirDestinationSave: String; DirForSave, KeyForSave: TListBox;
  Var ZipMaster: TZipMaster; CompressLevel: Integer);
Procedure MFCreateListOfSave;
Procedure MFFreeListOfSave;
Procedure MFScanForSave(MyListBox: TListBox; PathOfSave: String; Silent: Boolean = False);
Procedure MFSetRestoreSettings(ACleanDirBeforeRestore, AEraseFileOnRestore,
  ACleanKeyBeforeRestore, AEraseKeyOnRestore: Boolean);
Procedure MFRestoreSave(MyItemSelected: Integer; PathOfSave: String; Var ZipMaster: TZipMaster);
Function MFDelSaveSelected(SaveSelected: Integer; PathOfSave: String; Warn: Boolean = True): Boolean;
Function MFDelAllSave(PathOfSave: String): Boolean;
Procedure MFCheckUpdate(MyItemIndex: Integer);
Procedure MFCheckUpdateAtStart(MyItemIndex: Integer; UpdateAtStart: Boolean);
Procedure MFSetStatusProcedure(ASetlStatus: TSetlStatus; ASetpbStatusPosition, ASetpbStatusTotalPosition: TSetpbStatusPosition);
Procedure CancelNow;
Procedure SetUseCorbeille(State: Boolean);

Implementation

{$R skins.RES}
{$R updater.RES}

Type
  TRestoreSettings = Record
    CleanDirBeforeRestore, EraseFileOnRestore, CleanKeyBeforeRestore, EraseKeyOnRestore: Boolean;
  End;

Var
  ListOfSave:               TStringList;
  RestoreSettings:          TRestoreSettings;
  SetlStatus:               TSetlStatus;
  SetpbStatusPosition:      TSetpbStatusPosition;
  SetpbStatusTotalPosition: TSetpbStatusPosition;
  Cancel, UseCorbeille:     Boolean;

//vrifie si un dossier existe et sinon avertit l'utilisateur
Function MFDirectoryExists(MyDir: String): Boolean;
Begin
  //si aucun dossier n'est pass en paramtre alors on ne teste pas on renvoie directement false
  If MyDir = '' Then
    Result := False
  Else
  If Not SysUtils.DirectoryExists(MyDir) Then // si le dossier n'existe pas on prvient l'utilisateur
  Begin
    Application.MessageBox(PChar(MSGDirDontExist),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Result := False;
  End
  Else
    Result := True; //le dossier existe on renvoie true
End;

//permet de dterminer quelle est la cl root d'une chane passe en paramtre
//renvoie la clef trouve sinon 0 et l'enlve de la chane passe en paramtre
Function MFFindRegistryRootKey(Var MyKey: String): Cardinal;
Var
  RootKey:    String;
  PosSlashes: Integer;
Begin
  Result     := 0; //si la cl root n'est pas trouv on renvoie zro
  PosSlashes := Pos('\', MyKey);
  RootKey    := Copy(MyKey, 0, PosSlashes - 1); //on recupere la clef root
  MyKey      := Copy(MyKey, PosSlashes + 1, Length(MyKey) - 1); //on efface la clef root de mykey
  If RootKey = 'HKEY_CLASSES_ROOT' Then
    Result := HKEY_CLASSES_ROOT
  Else
  If RootKey = 'HKEY_CURRENT_USER' Then
    Result := HKEY_CURRENT_USER
  Else
  If RootKey = 'HKEY_LOCAL_MACHINE' Then
    Result := HKEY_LOCAL_MACHINE
  Else
  If RootKey = 'HKEY_USERS' Then
    Result := HKEY_USERS
  Else
  If RootKey = 'HKEY_PERFORMANCE_DATA' Then
    Result := HKEY_PERFORMANCE_DATA
  Else
  If RootKey = 'HKEY_CURRENT_CONFIG' Then
    Result := HKEY_CURRENT_CONFIG
  Else
  If RootKey = 'HKEY_DYN_DATA' Then
    Result := HKEY_DYN_DATA;
End;

//on vrifie si une cl registres existe et sinon si le paramtre ShowMsg est  true on avertit l'utilisateur
Function MFKeyExists(MyKey: String; ShowMsg: Boolean = True): Boolean;
Begin
  //si aucune cl n'est passe en paramtre alors on ne teste pas on renvoie directement false
  If MyKey = '' Then
  Begin
    Result := False;
    Exit;
  End;

  With TRegistry.Create Do
  Begin
    Try
      RootKey := MFFindRegistryRootKey(MyKey); //on recupere la rootkey et on l'enleve de mykey
      If (RootKey = 0) Or Not KeyExists(MyKey) Then
      Begin
        If ShowMsg Then
        Begin
          Application.MessageBox(PChar(MSGKeyDontExist),
            PChar(MSGError),
            MB_OK Or MB_ICONERROR);
        End;
        Result := False;
      End
      Else
        Result := True;
    Finally
      Free;
    End;
  End;
End;

//-code copi provenant de:
//-http://www.developpez.net/forums/d2680/environnements-developpement/delphi/composants/probleme-tregistry-savekey/
Function _AdjustTokenPrivileges(TokenHandle: THandle; DisableAllPrivileges: BOOL; NewState: PTokenPrivileges;
  BufferLength: DWORD; PreviousState: Integer; ReturnLength: Integer): BOOL; Stdcall; External advapi32 Name 'AdjustTokenPrivileges';

// Fonction de mise  jour des droits utilisateurs avec NT/2000/XP
Function LoggedSetBackupPrivilege(hProcess: THANDLE; bEnable: Boolean): Boolean;
Var
  Info:  TTokenPrivileges;
  Token: THandle;
  Res:   Boolean;
Const
  SE_BACKUP_NAME = 'SeBackUpPrivilege';
Begin
  // Ouverture des droits
  Res := OpenProcessToken(hProcess, TOKEN_ADJUST_PRIVILEGES, Token);
  If Not Res Then
  Begin
    Application.MessageBox(PChar(MSGCannotOpenPrivilege),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Result := False;
    Exit;
  End;

  // Mise  True ou False du droit
  Info.PrivilegeCount := 1;
  If bEnable Then
    Info.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED
  Else
    Info.Privileges[0].Attributes := 0;

  // Obtention du LUID.
  Res := LookupPrivilegeValue(NIL, SE_BACKUP_NAME, Info.Privileges[0].Luid);
  If Not Res Then
  Begin
    Application.MessageBox(PChar(MSGCannotFindPrivilegeValue),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Result := False;
    Exit;
  End;

  // Modification du privilge
  Res := _AdjustTokenPrivileges(Token, False, @Info, 0, 0, 0);
  If Not Res Then
  Begin
    Application.MessageBox(PChar(Format(MSGCannotModifyPrivilege, [IntToStr(GetLastError)])),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Result := False;
    Exit;
  End
  Else
  Begin
    If GetLastError <> ERROR_SUCCESS Then
    Begin
      Application.MessageBox(PChar(MSGCannotTakePrivilege),
        PChar(MSGError),
        MB_OK Or MB_ICONERROR);

      Result := False;
      Exit;
    End;
  End;

  CloseHandle(Token);
  Result := True;
End;

Function DeleteFileToRecycleBin(AFileName: String): Boolean;
Var
  fos: TSHFileOpStruct;
Begin
  FillChar(fos, SizeOf(fos), 0);
  With fos Do
  Begin
    wFunc  := FO_DELETE;
    pFrom  := PChar(AFileName + #0);
    fFlags := FOF_ALLOWUNDO Or FOF_NOCONFIRMATION Or FOF_SILENT;
  End;
  Result := (0 = ShFileOperation(fos));
End;

// Procdure pour compter les fichiers
Function CompteFichiers(Chemin: String): Integer;
Var
  S: TSearchRec;
Begin
  Chemin := IncludeTrailingPathDelimiter(Chemin);

  Result := 0;
  // Recherche de la premire entre du rpertoire
  If FindFirst(Chemin + '*.*', faAnyFile, S) = 0 Then
  Begin
    Repeat
      // Il faut absolument dans le cas d'une procdure rcursive ignorer
      // les . et .. qui sont toujours placs en dbut de rpertoire
      // Sinon la procdure va boucler sur elle-mme.
      If (S.Name <> '.') And (s.Name <> '..') Then
      Begin
        If (S.Attr And faDirectory) <> 0
          // Dans le cas d'un sous-repertoire on appelle la mme procdure
        Then
          Result := Result + CompteFichiers(Chemin + S.FindData.cFileName)
          // Sinon on compte simplement le fichier
        Else
          Inc(Result);
      End;
    // Recherche du suivant
    Until FindNext(S) <> 0;
    FindClose(S);
  End;
End;
//-fin code copi

// permet d'effacer un dossier ainsi que les sous dossiers et fichiers inclus dedans
// en affichant la progressionb et en appelant Processmessages toute les seconde
// pour eviter une apparence bloqu.
Function DelDirWithProgress(Dir: String): Boolean;
Var
  iIndex, LastRefresh, NowIts: Integer;
  SearchRec:                   TSearchRec;
  sFileName:                   String;
Begin
//  SetlStatus('Suppression du dossier existant avant restauration');
  SetlStatus('Effacement de: ' + Dir);
  LastRefresh := DateTimeToUnix(Now);

  If UseCorbeille Then
  Begin
  { TODO 2: crer un thread qui appelle processmessage? car la sa apparait bloqu pendant traitement}
    Result := DeleteFileToRecycleBin(ExcludeTrailingPathDelimiter(Dir));
    Exit;
  End
  Else
  Begin
  { DONE 2: possibillit de rglage, on efface le dossier directement avec code ci-dessous
  ou on met dans la corbeille comme actuellement ci-dessus.}
    Dir    := IncludeTrailingPathDelimiter(Dir);
    iIndex := FindFirst(Dir + '*.*', faAnyFile, SearchRec);
    If iIndex = 0 Then
    Begin
      Repeat
        sFileName := Dir + SearchRec.Name;
//      SetlStatus('Effacement de: ' + sFileName);
        If DirectoryExists(sFileName){SearchRec.Attr = faDirectory} Then //si l'objet est un dossier
        Begin
          If (SearchRec.Name <> '') And
            (SearchRec.Name <> '.') And
            (SearchRec.Name <> '..') Then //si l'objet n'est pas vide et ce n'est pas les dossiers systmes pour remonte
            DelDirWithProgress(sFileName); //alors on l'efface
        End
        Else
        Begin
          If SearchRec.Attr <> faArchive Then
            FileSetAttr(sFileName, faArchive);
          DeleteFile(sFileName);
        End;
      //si une seconde est pass on appelle Application.ProcessMessages;
        NowIts := DateTimeToUnix(Now);
        If NowIts > LastRefresh Then
        Begin
          LastRefresh := NowIts;
          Application.ProcessMessages;
        End;
        iIndex := FindNext(SearchRec); //on cherche le suivant
      Until iIndex <> 0;
      FindClose(SearchRec); //on ferme le dossier
      Result := RemoveDir(ExtractFileDir(Dir)); //on efface le dossier de base qui est maintenant vide normalement
    End
    Else
      Result := False;
  End;
End;

//permet de restaurer un dossier zip  l'emplacement spcifi
Procedure MFRestoreThisDir(ZipFile, DestionationPath: String; Var ZipMaster: TZipMaster);
Begin
  If DestionationPath = '' Then
    Exit;
  DestionationPath := IncludeTrailingPathDelimiter(DestionationPath);

  //on vrifie que le dossier existe sinon on tente de le crer
  If DirectoryExists(DestionationPath) Then
  Begin
    //si dans les rglages d'utilisateurs a choisi d'effacer le dossiers avant de restaurer
    If RestoreSettings.CleanDirBeforeRestore Then
    Begin
      //si on n'arrive pas  effacer le dossier et ses sous dossier avant de le recrer
      //on avertit l'utilisateur et on quitte la procdure
      If Not DelDirWithProgress(DestionationPath) Or Not CreateDir(DestionationPath) Then
      Begin
        Application.MessageBox(PChar(Format(MSGCannotCreateRestoreDir, [DestionationPath, ZipFile])),
          PChar(MSGError),
          MB_OK Or MB_ICONERROR);
        Exit;
      End;
      SetlStatus('Cration du dossier de destination');
    End;
  End
  Else
  Begin
    //si on n'arrive pas  crer le dossier et ses dossiers parents si il n'existe pas
    //on avertit l'utilisateur et on quitte la procdure
    If Not ForceDirectories(DestionationPath) Then
    Begin
      Application.MessageBox(PChar(Format(MSGCannotCreateRestoreDir, [DestionationPath, ZipFile])),
        PChar(MSGError),
        MB_OK Or MB_ICONERROR);
      Exit;
    End;
    SetlStatus('Cration du dossier de destination');
  End;

  //on dcompresse le fichier zip pass en paramtres de la procdure
  ZipMaster.Cancel  := False;
  ZipMaster.ErrCode := 0;
  ZipMaster.FSpecArgs.Clear;
  ZipMaster.ZipFilename := ZipFile;
  ZipMaster.ExtrBaseDir := DestionationPath;
  SetlStatus('Restoration dossier: ' + DestionationPath);
  ZipMaster.Extract;
  If Not Cancel And (ZipMaster.SuccessCnt < 1) Then
  Begin
    Application.MessageBox(PChar(Format(MSGErrorOnExtractZip, [ZipFile])),
      PChar(MSGWarning),
      MB_OK Or MB_ICONWARNING);
  End;
End;

//permet de restaurer un fichier .reg dans la base de registre
Procedure MFRestoreThisKey(KeyFile, DestinationKey: String);
Var
  tmpKey: String;
Begin
  If DestinationKey = '' Then
    Exit;

  With TRegistry.Create Do //on cr un objet registre
  Begin
    Try
      tmpKey  := DestinationKey; //on met le nom de la cl entire
      RootKey := MFFindRegistryRootKey(tmpKey); //on recupere la rootkey et on l'enleve de tmpKey
      If RootKey = 0 Then
      Begin
        { TODO 2: ajouter message erreur clefs incorrect}
        Exit;
      End;

      //l'utilisateur veut effacer la cl avant de restaurer
      If RestoreSettings.CleanKeyBeforeRestore Then
      Begin
        SetlStatus('Effacement clef: ' + DestinationKey);
        //on efface la clef et on attend la fin de l'opration avant de continuer
        WaitExec(SystemDirectory + 'reg.exe', 'delete "' + DestinationKey + '" /f');
      End;

      //la cl existe et l'utilisateur ne veut pas l'craser alors on quitte la procdure
      If Not RestoreSettings.EraseKeyOnRestore And KeyExists(tmpKey) Then
      Begin
        Exit;
      End;

      If Cancel Then
        Exit;

      SetlStatus('Restoration clef: ' + DestinationKey);
      //on importe la cl dans la base de registre
      WaitExec(SystemDirectory + 'reg.exe', 'IMPORT "' + KeyFile + '.reg"');
    Finally
      Free; //on libre l'objet registre
    End;
  End;
End;

//------- exported:

//permet d'afficher la bote de slection de dossier avec la description passe en paramtre
//et insre le rsultat dans le tedit plac en paramtre, optionnellement on peut ajouter des options
//pour la bote de dialogue par exemple pour afficher le bouton nouveau dossier
Function MFSelectDirectory(Describe: String; MyEdit: TEdit; Options: TSelectDirExtOpts = [sdNewUI]): Boolean;
Var
  DirForAddToSaveList: String;
Begin
  DirForAddToSaveList := MyEdit.Text;
  If SelectDirectory(Describe, '', DirForAddToSaveList, Options) Then
  Begin
    MyEdit.Text := DirForAddToSaveList;
    Result      := True;
  End
  Else
    Result := False;
End;

//efface de la liste passe en paramtre les dossiers qui n'existent pas
Procedure MFDelEmptyDir(MyListBox: TListBox);
Var
  I: Integer;
Begin
  If MyListBox.Items.Count = 0 Then
    Exit;
  //on passe les lments de la liste 1  1 an partant du dernier jusqu'au 1er
  //car lorsqu'on efface un lment si ce n'est pas le dernier de la liste
  //alors toute la liste se dcale pour pas avoir d'lment vide
  I := MyListBox.Items.Count - 1;
  Repeat
    If Not SysUtils.DirectoryExists(MyListBox.Items[I]) Then
      MyListBox.Items.Delete(I);
    Dec(I);
  Until I = -1;
End;

//efface de la liste passe en paramtre les cls registres qui n'existent pas
Procedure MFDelEmptyKey(MyListBox: TListBox);
Var
  I: Integer;
Begin
  If MyListBox.Items.Count = 0 Then
    Exit;
  //on passe les lments de la liste 1  1 an partant du dernier jusqu'au 1er
  //car lorsqu'on efface un lment si ce n'est pas le dernier de la liste
  //alors toute la liste se dcale pour pas avoir d'lment vide
  I := MyListBox.Items.Count - 1;
  Repeat
    If Not MFKeyExists(MyListBox.Items[I], False) Then
      MyListBox.Items.Delete(I);
    Dec(I);
  Until I = -1;
End;

//efface de la liste passe en paramtre la ligne slectionne
Procedure MFDelSelectedItem(MyListBox: TListBox);
Begin
  If MyListBox.ItemIndex <> -1 Then
    MyListBox.Items.Delete(MyListBox.ItemIndex);
End;

//permet de vrifier si le dossier contenu dans le tedit passer en paramtre existe
//et si oui l'ajoute dans la tlistbox passer en paramtre
Procedure MFAddDirInListBox(MyEdit: TEdit; MyListBox: TListBox);
Begin
  If Not MFDirectoryExists(MyEdit.Text) Then
    Exit;

  //si le dossier est dj dans la liste on avertit l'utilisateur
  If MyListBox.Items.IndexOf(MyEdit.Text) <> -1 Then
  Begin
    Application.MessageBox(PChar(MSGDirAlreadyInList),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
  End
  Else
  Begin
    MyListBox.Items.Add(MyEdit.Text); //sinon on ajoute le dossier dans la liste
                                      //    MyEdit.Clear;
  End;
End;

//permet de remplacer dans la tlistbox passait en paramtre la ligne slectionne
//par le dossier contenu dans le tedit pass en paramtresi il existe
Procedure MFReplaceDirInListBox(MyEdit: TEdit; MyListBox: TListBox);
Begin
  If Not MFDirectoryExists(MyEdit.Text) Then
    Exit;

  If MyListBox.ItemIndex <> -1 Then //si une ligne est slectionne
    MyListBox.Items.Strings[MyListBox.ItemIndex] := MyEdit.Text; //on la remplace
End;

//ajoute une cl de registre dans la tlistbox pass en paramtre
//la cl de registre ajout est contenue dans le tedit pass en paramtre
Procedure MFAddKeyInListBox(MyEdit: TEdit; MyListBox: TListBox);
Begin
  If Not MFKeyExists(MyEdit.Text) Then //si la cl existe pas on arrte
    Exit;

  //si la cl est dj dans la liste on avertit l'utilisateur
  If MyListBox.Items.IndexOf(MyEdit.Text) <> -1 Then
  Begin
    Application.MessageBox(PChar(MSGKeyAlreadyInList),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
  End
  Else
    MyListBox.Items.Add(MyEdit.Text); //sinon on ajoute la cl dans la liste
End;

//permet de remplacer une cl de registre dans la tlistbox pass en paramtre
//la cl contenue dans le tedit doit exister
//et une ligne doit tre slectionne dans le tlistbox
Procedure MFReplaceKeyInListBox(MyEdit: TEdit; MyListBox: TListBox);
Begin
  If Not MFKeyExists(MyEdit.Text) Then
    Exit;

  If MyListBox.ItemIndex <> -1 Then // si une ligne est slectionne
    MyListBox.Items.Strings[MyListBox.ItemIndex] := MyEdit.Text;
End;

//permet d'appliquer le skins slectionns  l'application
Procedure MFSetSkinSelected(Var MySkinData: TSkinData; MyItemIndex: Integer);
Var
  SkinName: String;
Begin
  If MyItemIndex = 0 Then //la valeur slectionne est sans skin on le dsactive
  Begin
    MySkinData.Active := False;
  End
  Else //sinon on applique le skins slectionns
  Begin
    SkinName := 'skin' + IntToStr(MyItemIndex);
    If ExtractOfRes(SkinName, TempDirectory + SkinName + '.skn') Then
    Begin
      MySkinData.LoadFromFile(TempDirectory + SkinName + '.skn');
      MySkinData.Active := True;
      MySkinData.DoSkinChanged;
      DeleteFile(TempDirectory + SkinName);
    End
    Else
      Application.MessageBox(PChar(MSGCannotLoadSkin), PChar(MSGError),
        MB_ICONERROR Or MB_OK);
  End;
End;

//permet de crer une sauvegarde des dossiers et cls contenus dans les tlistbox pass en paramtre
Procedure MFStartSave(NameForSave, DirDestinationSave: String; DirForSave, KeyForSave: TListBox;
  Var ZipMaster: TZipMaster; CompressLevel: Integer);
Var
  I, KeyAndFileSaved, NbFileInDir, IdResultMsgBox: Integer;
  tmpKey, tmpName, tmpParam, UnikNameForSave:      String;
  IniFile:                                         TMemIniFile;
  DTStart:                                         TDateTime;
Begin
  //si les tlistbox sont vides on arrte
  If (DirForSave.Items.Count = 0) And (KeyForSave.Items.Count = 0) Then
    Exit;

  //on verifie que les procedure d'affichage de progression fonctionne
  If Not Assigned(SetlStatus) Or Not Assigned(SetpbStatusPosition) Or Not Assigned(SetpbStatusTotalPosition) Then
    Exit;

  //variable qui contient l'heure de dpart enfin d'afficher la dure de l'opration
  DTStart := Now;
  //variable qui dtermine si on doit arrter les oprations en cours de route
  Cancel  := False;

  //on met la barre de progression total  zro avec un maximum
  SetpbStatusTotalPosition(0, DirForSave.Items.Count + KeyForSave.Items.Count);

  //on verifie que le dossier de destination existe sinon on tente la cration
  //si la cration choue on avertit l'utilisateur et on arrte
  SetlStatus('Cration du dossier de destination');
  DirDestinationSave := IncludeTrailingPathDelimiter(DirDestinationSave);
  If Not DirectoryExists(DirDestinationSave) Then
  Begin
    If Not ForceDirectories(DirDestinationSave) Then
    Begin
      Application.MessageBox(PChar(MSGCannotCreateDir),
        PChar(MSGError),
        MB_OK Or MB_ICONERROR);
      Exit;
    End;
  End;

  SetlStatus('Chargement du module de compression');
  //on vrifie que la dll zip est charg sinon on essaye de la charger
  //si le chargement choue on avertit l'utilisateur et on arrte
  If LoadDllZipNow(ZipMaster) = DLLZIP_ERREXTRACT Then
  Begin
    Application.MessageBox(PChar(MsgErrZipExtract),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Exit;
  End;
  //on applique les rglages pour la compression
  ZipMaster.AddCompLevel := CompressLevel;
  ZipMaster.ZipComment   := MSGZipComment; // on ajoute un commentaire  l'archive cre

  //on cre un nom unique pour la sauvegarde compos du nom choisi par l'utilisateur et
  //du nombre de secondes  partir de now
  UnikNameForSave := NameForSave + '-' + IntToStr(DateTimeToUnix(Now));
  //chaque fois qu'un lment sera sauvegard on incrmentera cette variable
  //afin de savoir  la fin de la sauvegarde si des lments ont t enregistr
  KeyAndFileSaved := 0;

  //on cree un .ini qui va contenir les details de la sauvegarde pour la restoration
  IniFile := TMemIniFile.Create(DirDestinationSave + UnikNameForSave + '.ini');
  Try
    //on sauvegarde les dossiers un  un dans un .zip
    For I := 0 To DirForSave.Items.Count - 1 Do
    Begin
      //si l'utilisateur choisit d'arrter on sort de la boucle
      If Cancel Then
        Break;

      DirForSave.Items[I] := IncludeTrailingPathDelimiter(DirForSave.Items[I]);
      If SysUtils.DirectoryExists(DirForSave.Items[I]) Then
      Begin
        SetlStatus('Vrification dossier: ' + DirForSave.Items[I]);
        //bug delphizip nombre fichier suprieur  65534 certain ne seront pas sauvegard
        //on propose  l'utilisateur de rduir taille, le sauter ou tout annuler
        Repeat
          NbFileInDir := CompteFichiers(DirForSave.Items[I]);
          If NbFileInDir > 65500 Then
          Begin
            IdResultMsgBox := Application.MessageBox(PChar(Format(MSGToMuchBig, [DirForSave.Items[I], NbFileInDir])), PChar(MSGWarning),
              MB_ICONWARNING Or MB_ABORTRETRYIGNORE Or MB_DEFBUTTON2);
            If (IdResultMsgBox = idIgnore) Or (IdResultMsgBox = idAbort) Then
            Begin
              NbFileInDir := 0; //on sort de la boucle de vrif avant de faire quoi que se soit
            End
            Else
            Begin
              NbFileInDir := CompteFichiers(DirForSave.Items[I]); //l'utilisateur ressaye on recompte les fichiers.
            End;
          End;
        Until NbFileInDir <= 65500;
        If IdResultMsgBox = idAbort Then
        Begin
          Cancel := True; //on annule la sauvegarde
          Break;          //on sort de la boucle for
        End
        Else
        If IdResultMsgBox <> idIgnore Then
        Begin
          SetlStatus('Sauvegarde dossier: ' + DirForSave.Items[I]);
          ZipMaster.FSpecArgs.Clear;
          ZipMaster.FSpecArgs.Add('*.*'); //oui il veut tout
          ZipMaster.ZipFilename := DirDestinationSave + UnikNameForSave + IntToStr(I) + '.zip';//on affecte le nom de la sauvegarde  l'objet qui va la crer
          ZipMaster.RootDir     := DirForSave.Items[I]; //le dossier de base de la sauvegarde est le dossier des profils
          ZipMaster.Add;                  //on dmarre la cration de la sauvegarde
                                        //si il y a eu une erreur durant la compression on avertit l'utilisateur
          If Not Cancel And (ZipMaster.SuccessCnt < 1) Then
          Begin
            Application.MessageBox(PChar(Format(MSGCannotCreateZip, [DirForSave.Items[I]])),
              PChar(MSGError),
              MB_OK Or MB_ICONERROR);
            continue;
          End
          Else //sinon on ajoute une entre dans le fichier ini pour la restauration et on incrmente KeyAndFileSaved
          Begin
            IniFile.WriteString('Dirs', UnikNameForSave + IntToStr(I), DirForSave.Items[I]);
            Inc(KeyAndFileSaved);
          End;
        End;
      End;
      SetpbStatusTotalPosition(I + 1);
    End;

      //si l'utilisateur choisit d'arrter on sort de la procdure
    If Cancel Then
      Exit;

    // Il faut les droits de 'BACKUP' pour utiliser SaveKey
    LoggedSetBackupPrivilege(GetCurrentProcess, True);

    If KeyForSave.Items.Count > 0 Then
      SetpbStatusPosition(0, 1);
    //on sauvegarde une  une les clefs registre contenu dans la listbox
    For I := 0 To KeyForSave.Items.Count - 1 Do
    Begin
      //si l'utilisateur choisit d'arrter on sort de la boucle
      If Cancel Then
        Break;

      SetlStatus('Sauvegarde clefs: ' + KeyForSave.Items[I]);
      With TRegistry.Create Do //ont cr un objet registre
      Begin
        Try
          tmpKey  := KeyForSave.Items[I];//on met le nom de la cl
          RootKey := MFFindRegistryRootKey(tmpKey); //on recupere la rootkey et on l'enleve de tmpKey
          If (RootKey <> 0) And KeyExists(tmpKey) Then //si la cl existe on tente de la sauvegarder
          Begin
            tmpName  := UnikNameForSave + IntToStr(I); //nom du fichier de sauvegarde
            tmpParam := 'EXPORT "' + KeyForSave.Items[I] + '" "' + (DirDestinationSave + tmpName + '.reg') + '"';
            If (WaitExec(SystemDirectory + 'reg.exe', tmpParam) = 0) And Not Cancel {SaveKey(tmpKey, DirDestinationSave + tmpName)} Then //si la sauvegarde russie reg.exe renvoie 0 sinon 1
            Begin
              IniFile.WriteString('Keys', tmpName, KeyForSave.Items[I]); //on ajoute une entre dans le ini pour la restauration
              Inc(KeyAndFileSaved); //et on incrmente KeyAndFileSaved
            End;
          End
          Else
          Begin
            Application.MessageBox(PChar(Format(MSGCannotCreateReg, [KeyForSave.Items[I]])),
              PChar(MSGError),
              MB_OK Or MB_ICONERROR);
          End;
        Finally
          Free; //on libre l'objet registre
        End;
      End;
      SetpbStatusPosition(1, 1);
      SetpbStatusTotalPosition(DirForSave.Items.Count + I + 1);
      Application.ProcessMessages;
    End;

    // On supprime ensuite les droits de BackUp donns plus haut
    LoggedSetBackupPrivilege(GetCurrentProcess, False);

    //si KeyAndFileSaved est suprieur  zro c'est que des lments ont t sauvegards
    //alors on enregistre dans le fichier ini le nom de la sauvegarde
    //ainsi que la date et le nom unique
    If Not Cancel And (KeyAndFileSaved > 0) Then
    Begin
      SetlStatus('Finalisation sauvegarde');
      IniFile.WriteString('General', 'Name', NameForSave);
      IniFile.WriteString('General', 'Date', DateTimeToStr(Now));
      IniFile.UpdateFile; //on met  jour le fichier sur le disque dur
      Application.MessageBox(PChar(Format(MSGEndSave, [FormatDateTime('hh "H" nn "Min" ss "Sec"', Now - DTStart)])),
        PChar(MSGInformation),
        MB_OK Or MB_ICONINFORMATION);
    End;
  Finally
    //si l'utilisateur a choisi d'annuler alors on supprime les fichiers dj sauvegards
    If Cancel Then
    Begin
      For I := 0 To DirForSave.Items.Count - 1 Do
      Begin
        tmpName := DirDestinationSave + UnikNameForSave + IntToStr(I) + '.zip';
        If FileExists(tmpName) Then
          DeleteFile(tmpName);
      End;
      For I := 0 To KeyForSave.Items.Count - 1 Do
      Begin
        tmpName := DirDestinationSave + UnikNameForSave + IntToStr(I) + '.reg';
        If FileExists(tmpName) Then
          DeleteFile(tmpName);
      End;
    End;
    IniFile.Free;
  End;
End;

//permettent de crer la liste qui contient le nom des fichiers de sauvegarde trouve
//lors du scan du dossier de sauvegarde. procdure appele  l'ouverture de la forme principale
Procedure MFCreateListOfSave;
Begin
  ListOfSave := TStringList.Create;
End;

//permettent de librer la liste qui contient le nom des fichiers de sauvegarde trouve
//lors du scan du dossier de sauvegarde. procdure appele  la fermeture de la forme principale
Procedure MFFreeListOfSave;
Begin
  ListOfSave.Free;
End;

//permet de chercher dans le dossier pass en paramtre les fichiers ini de sauvegarde
//et affich dans le tlistbox passer en paramtre le nom des sauvegardes ainsi que leur date
//et ajout dans ListOfSave le nom unique des fichiers contenus dans le tlistbox
//silent permet de scanner sans afficher de message
Procedure MFScanForSave(MyListBox: TListBox; PathOfSave: String; Silent: Boolean = False);
Var
  MySearch:         TSearchRec;
  tmpName, tmpDate: String;
Begin
  //on verifie que le dossier existe sinon on avertit l'utilisateur et on arrte
  PathOfSave := IncludeTrailingPathDelimiter(PathOfSave);
  If Not SysUtils.DirectoryExists(PathOfSave) Then
  Begin
    If Not Silent Then
      Application.MessageBox(PChar(Format(MSGNoSaveDir, [PathOfSave])),
        PChar(MSGError),
        MB_OK Or MB_ICONERROR);
    Exit;
  End;

  //on vide les deux listes qui contiennent les dtails et nom des fichiers ini
  MyListBox.Items.Clear;
  ListOfSave.Clear;

  //on cherche les .ini
  If FindFirst(PathOfSave + '*.ini', faAnyFile, MySearch) = 0 Then
  Begin
    Try
      Repeat
        With TIniFile.Create(PathOfSave + MySearch.FindData.cFileName) Do
        Begin
          Try
            tmpName := ReadString('General', 'Name', '');
            tmpDate := ReadString('General', 'Date', '');
            If (tmpName <> '') And (tmpDate <> '') Then
            Begin
              MyListBox.Items.Add(tmpName + ' ' + tmpDate);
              ListOfSave.Add(MySearch.FindData.cFileName);
            End;
          Finally
            Free;
          End;
        End;
      //on boucle pour tout lister le repertoire
      Until FindNext(MySearch) <> 0;
    Finally
      FindClose(MySearch);
    End;
  End;

  //si aucune sauvegarde trouv
  If MyListBox.Items.Count = 0 Then
  Begin
    If Not Silent Then //si on peut afficher des messages
      Application.MessageBox(PChar(MSGNoSaveFound),
        PChar(MSGInformation),
        MB_OK Or MB_ICONINFORMATION);
  End;
End;

//permet de passer  cette unit les rglages actuels de restauration des sauvegardes
Procedure MFSetRestoreSettings(ACleanDirBeforeRestore, AEraseFileOnRestore,
  ACleanKeyBeforeRestore, AEraseKeyOnRestore: Boolean);
Begin
  RestoreSettings.CleanDirBeforeRestore := ACleanDirBeforeRestore;
  RestoreSettings.EraseFileOnRestore    := AEraseFileOnRestore;
  RestoreSettings.CleanKeyBeforeRestore := ACleanKeyBeforeRestore;
  RestoreSettings.EraseKeyOnRestore     := AEraseKeyOnRestore;
End;

//permet de restaurer le fichier ini slectionn dans la liste des sauvegardes trouves
Procedure MFRestoreSave(MyItemSelected: Integer; PathOfSave: String; Var ZipMaster: TZipMaster);
Var
  FileNameIni:                String;
  IniFile:                    TIniFile;
  DirToRestore, KeyToRestore: TStringList;
  I:                          Integer;
  DTStart:                    TDateTime;
Begin
  //si la ligne slectionne ne fait pas partie de la liste on quitte la procdure
  If (MyItemSelected < 0) Or (MyItemSelected > (ListOfSave.Count - 1)) Then
    Exit;

  //on verifie que les procedure d'affichage de progression fonctionne
  If Not Assigned(SetlStatus) Or Not Assigned(SetpbStatusPosition) Or Not Assigned(SetpbStatusTotalPosition) Then
    Exit;

  //variable qui contient l'heure de dpart enfin d'afficher la dure de l'opration
  DTStart := Now;
  //variable qui dtermine si on doit arrter les oprations en cours de route
  Cancel  := False;

  PathOfSave  := IncludeTrailingPathDelimiter(PathOfSave);
  FileNameIni := PathOfSave + ListOfSave.Strings[MyItemSelected];

  //si le fichier  restaurer n'existe pas on avertit l'utilisateur et on quitte la procdure
  If Not FileExists(FileNameIni) Then
  Begin
    Application.MessageBox(PChar(Format(MSGNoSaveFileFound, [FileNameIni])),
      PChar(MSGError),
      MB_OK Or MB_ICONERROR);
    Exit;
  End;

  SetlStatus('Chargement information de sauvegarde');
  IniFile      := TIniFile.Create(FileNameIni);
  DirToRestore := TStringList.Create;
  KeyToRestore := TStringList.Create;
  Try
    //si dans le fichier ini il y a des dossiers  restaurer
    If IniFile.SectionExists('Dirs') Then
    Begin
      //on lit la liste des dossier zip  restaurer
      IniFile.ReadSection('Dirs', DirToRestore);
    End;

    If Cancel Then
      Exit;

    //si dans le fichier ini il y a des cls  restaurer
    If IniFile.SectionExists('Keys') Then
    Begin
      //on lit la liste des cls  restaurer
      IniFile.ReadSection('Keys', KeyToRestore);
    End;

    If Cancel Then
      Exit;

    If (KeyToRestore.Count > 0) Or (DirToRestore.Count > 0) Then
    Begin
      SetpbStatusTotalPosition(0, KeyToRestore.Count + DirToRestore.Count);

      If DirToRestore.Count > 0 Then //si notre liste comptent au moins un dossier  restaurer
      Begin
        //on vrifie que la dll zip est charg sinon on essaye de la charger
        //si le chargement choue on avertit l'utilisateur et on arrte
        SetlStatus('Chargement du module de dcompression');
        If LoadDllZipNow(ZipMaster) = DLLZIP_ERREXTRACT Then
        Begin
          Application.MessageBox(PChar(MsgErrZipExtract),
            PChar(MSGError),
            MB_OK Or MB_ICONERROR);
          Exit;
        End;
        //on applique les rglages pour la dcompression
        If RestoreSettings.EraseFileOnRestore Then
          ZipMaster.ExtrOptions := ZipMaster.ExtrOptions + [ExtrOverWrite]
        Else
          ZipMaster.ExtrOptions := ZipMaster.ExtrOptions + [ExtrOverWrite];

        //on restore un  un les dossiers
        For I := 0 To DirToRestore.Count - 1 Do
        Begin
          If Cancel Then
            Break;
          MFRestoreThisDir(PathOfSave + DirToRestore.Strings[I] + '.zip',
            IniFile.ReadString('Dirs', DirToRestore.Strings[I], ''), ZipMaster);
          SetpbStatusTotalPosition(I + 1);
        End;
      End;

      If KeyToRestore.Count > 0 Then //si notre liste comptent au moins une cl  restaurer
      Begin
        SetpbStatusPosition(0, 1);
        //on restore une  une les cls
        For I := 0 To KeyToRestore.Count - 1 Do
        Begin
          If Cancel Then
            break;
          MFRestoreThisKey(PathOfSave + KeyToRestore.Strings[I],
            IniFile.ReadString('Keys', KeyToRestore.Strings[I], ''));
          SetpbStatusPosition(1);
          SetpbStatusTotalPosition(DirToRestore.Count + I + 1);
        End;
      End;

      If Not Cancel Then
        Application.MessageBox(PChar(Format(MSGEndRestore, [FormatDateTime('hh "H" nn "Min" ss "Sec"', Now - DTStart)])),
          PChar(MSGInformation),
          MB_OK Or MB_ICONINFORMATION);
    End;
  Finally
    IniFile.Free;
    DirToRestore.Free;
    KeyToRestore.Free;
  End;
End;

//permet d'effacer la sauvegarde slectionne en indiquant son numro
//et le dossier contenant les sauvegardes
Function MFDelSaveSelected(SaveSelected: Integer; PathOfSave: String; Warn: Boolean = True): Boolean;
Var
  Search:   TSearchRec;
  SaveName: String;
Begin
  Result := False;
  //si la ligne slectionne ne fait pas partie de la liste on quitte la procdure
  If (SaveSelected < 0) Or (SaveSelected > (ListOfSave.Count - 1)) Then
    Exit;

  If Application.MessageBox(PChar(MSGDelSaveSelected), PChar(MSGConfirm), MB_ICONQUESTION Or MB_YESNO Or MB_DEFBUTTON2) <> idYes Then
    Exit;

  PathOfSave := IncludeTrailingPathDelimiter(PathOfSave);
  SaveName   := ChangeFileExt(ListOfSave.Strings[SaveSelected], '');
  If FindFirst(PathOfSave + '*.*', faAnyFile, Search) = 0 Then
  Begin
    Try
      Repeat
        If AnsiPos(AnsiUpperCase(SaveName), AnsiUpperCase(Search.FindData.cFileName)) = 1 Then
        Begin
          If UseCorbeille Then
            DeleteFileToRecycleBin(PathOfSave + Search.FindData.cFileName)
          Else
            DeleteFile(PathOfSave + Search.FindData.cFileName);
          Result := True;
        End;
      Until FindNext(Search) <> 0;
    Finally
      FindClose(Search);
    End;
  End;
End;

//permet d'effacer toutes les sauvegardes contenue dans le dossier pass en paramtre
Function MFDelAllSave(PathOfSave: String): Boolean;
Var
  I: Integer;
Begin
  If Application.MessageBox(PChar(MSGDelAll), PChar(MSGConfirm), MB_ICONQUESTION Or MB_YESNO Or MB_DEFBUTTON2) <> idYes Then
    Result := False
  Else
  Begin
    For I := 0 To ListOfSave.Count - 1 Do
      MFDelSaveSelected(I, PathOfSave, False);
    Result := True;
  End;
End;

Procedure MFCheckUpdate(MyItemIndex: Integer);
Var
  SkinName: String;
Begin
  If Not StateUpdater Then
  Begin
    SkinName := 'skin' + IntToStr(MyItemIndex);
    If (MyItemIndex > 0) And ExtractOfRes(SkinName, TempDirectory + SkinName + '.skn') Then
      TUpdater := TThreadUpdater.Create(TempDirectory + SkinName + '.skn')
    Else
      TUpdater := TThreadUpdater.Create();
  End
  Else
    Application.MessageBox(PChar(MSGAlreadyCheck), PChar(MSGWait),
      MB_ICONINFORMATION Or MB_OK);
End;

Procedure MFCheckUpdateAtStart(MyItemIndex: Integer; UpdateAtStart: Boolean);
Var
  SkinName: String;
Begin
  If Not StateUpdater Then
  Begin
    SkinName := 'skin' + IntToStr(MyItemIndex);
    If (MyItemIndex > 0) And ExtractOfRes(SkinName, TempDirectory + SkinName + '.skn') Then
      TUpdater := TThreadUpdater.Create(TempDirectory + SkinName + '.skn', '', UpdateAtStart, False, True)
    Else
      TUpdater := TThreadUpdater.Create('', '', UpdateAtStart, False, True);
  End;
End;

Procedure MFSetStatusProcedure(ASetlStatus: TSetlStatus; ASetpbStatusPosition, ASetpbStatusTotalPosition: TSetpbStatusPosition);
Begin
  //on enregistre les deux procdures afin de les utiliser dans l'unit sans
  //avoir besoin de les passer en paramtre des fonctions de sauvegarde de dossiers etc.
  { TODO 1: il n'y a pas besoin de les dfinir  chaque appel de la fonction,  optimiser}
  SetlStatus               := ASetlStatus;
  SetpbStatusPosition      := ASetpbStatusPosition;
  SetpbStatusTotalPosition := ASetpbStatusTotalPosition;
End;

Procedure CancelNow;
Begin
  Cancel := True;
End;

Procedure SetUseCorbeille(State: Boolean);
Begin
  UseCorbeille := State;
End;

Initialization
  Cancel := False;

End.

