Unit ThreadUpdater;

Interface

Uses
  Classes
{$IFDEF MSWINDOWS}
  , Windows
{$ENDIF}
  , SysUtils, Forms, Dialogs, Utils, Wininet, IniFiles, ShellApi, Messages;

Type
  TThreadUpdater = Class(TThread)
  Private
    NewVersionName, UpdaterName, Language, Skin: String;
    UpdateAtStart, WarnNoNew, WaitBeforeStart, DownUpdate:           Boolean;
    Procedure SetName;
    Procedure CheckUpdate;
    Procedure SyncroMsgNew;
    Procedure SyncroMsgNoNew;
    Procedure StartUpdate(ActualVer, FileToDown, FileSize, FileMD5, Author, Mail, WebPage, Comment: String);
    Function DownAndReturnThis(URL: String): String;
  Protected
    Procedure Execute; Override;
  Public
    Constructor Create(ASkin: String = ''; ALanguage: String = '';
      AUpdateAtStart: Boolean= True; AWarnNoNew: Boolean = True; AWaitBeforeStart: Boolean = False);
    Destructor Destroy; Override;
  End;

Var
  TUpdater:     TThreadUpdater;
  StateUpdater: Boolean;

Implementation

{$IFDEF MSWINDOWS}
Type
  TThreadNameInfo = Record
    FType:     Longword;     // doit tre 0x1000
    FName:     PChar;        // pointeur sur le nom (dans l'espace d'adresse de l'utilisateur)
    FThreadID: Longword;    // ID de thread (-1=thread de l'appelant)
    FFlags:    Longword;    // rserv pour une future utilisation, doit tre zro
  End;
{$ENDIF}

{ uTUpdate }

Resourcestring
  MsgNew   = 'Une nouvelle version de Delphi Save And Restore est disponible! ( Version: %s )' + sLineBreak + 'Voulez-vous mettre  jour maintenant?';
  MsgNoNew = 'Votre version est  jour!';

Procedure TThreadUpdater.SetName;
{$IFDEF MSWINDOWS}
Var
  ThreadNameInfo: TThreadNameInfo;
{$ENDIF}
Begin
{$IFDEF MSWINDOWS}
  ThreadNameInfo.FType     := $1000;
  ThreadNameInfo.FName     := 'ThreadUpdater';
  ThreadNameInfo.FThreadID := $FFFFFFFF;
  ThreadNameInfo.FFlags    := 0;
  Try
    RaiseException($406D1388, 0, SizeOf(ThreadNameInfo) Div SizeOf(Longword), @ThreadNameInfo);
  Except
  End;
{$ENDIF}
End;

Constructor TThreadUpdater.Create(ASkin: String = ''; ALanguage: String = '';
      AUpdateAtStart: Boolean= True; AWarnNoNew: Boolean = True; AWaitBeforeStart: Boolean = False);
Begin
  StateUpdater := True;
  Inherited Create(False);
  FreeOnTerminate    := True;
  UpdaterName        := TempDirectory + ChangeFileExt(ExtractFileName(Application.ExeName), '_updater.exe');
  Language           := AnsiLowerCase(ALanguage);
  UpdateAtStart      := AUpdateAtStart;
  Skin               := ASkin;
  DownUpdate         := False;
  WarnNoNew          := AWarnNoNew;
  WaitBeforeStart    := AWaitBeforeStart;
End;

Destructor TThreadUpdater.Destroy;
Begin
  Inherited;
  StateUpdater := False;
End;

Procedure TThreadUpdater.Execute;
Var
  I: Integer;
Begin
  SetName;
  If WaitBeforeStart Then
  Begin
    I := 0;
    Repeat
      Sleep(100);
      Inc(I, 100);
    Until (I = 1500) Or Terminated;
  End;
  If Terminated Then
    Exit;
  If FileExists(UpdaterName) Then
    DeleteFile(UpdaterName);
  If UpdateAtStart Then
    CheckUpdate;
End;

Procedure TThreadUpdater.CheckUpdate;
Var
  Version:                                                                            Array[0..3, 0..3] Of Smallint;
  PageDown, ActualVer, FileToDown, FileSize, FileMD5, Author, Mail, WebPage, Comment: String;
  Search:                                                                             TStringList;
  PosComment:                                                                         Smallint;
Begin
  Search    := TStringList.Create;
  ActualVer := ApplicationVersion;
  Try
    Search.Delimiter     := '.';
    Search.QuoteChar     := #0;
    Search.DelimitedText := ActualVer;
    If Search.Count < 4 Then
      Exit;
    Version[0][0] := StrToInt(Search[0]);
    Version[0][1] := StrToInt(Search[1]);
    Version[0][2] := StrToInt(Search[2]);
    Version[0][3] := StrToInt(Search[3]);
  Finally
    Search.Free;
  End;
  Search := TStringList.Create;
  Try
    Search.Delimiter     := '#';
    Search.QuoteChar     := #0;
    PageDown             := DownAndReturnThis('http://www.toutenvrac.org/update/DelphiSaveAndRestore.ini');
    Search.DelimitedText := PageDown;
    If Search.Count < 16 Then
      Exit;
    NewVersionName := Search[1];
    FileToDown     := Search[3];
    FileSize       := Search[5];
    FileMD5        := Search[7];
    Author         := Search[9];
    Mail           := Search[11];
    WebPage        := Search[13];
    PosComment     := AnsiPos('comment#', PageDown) + 8;
    Comment        := copy(PageDown, PosComment, (Length(PageDown) - PosComment));
    //Search[15];
    //    if Search.Count > 15 then
    //    begin
    //      for I := 16 to Search.Count - 1 do
    //        Comment := Comment + Search[I];
    //    end;
    //    showmessage(comment);
  Finally
    Search.Free;
  End;
  Search := TStringList.Create;
  Try
    Search.Delimiter     := '.';
    Search.QuoteChar     := #0;
    Search.DelimitedText := NewVersionName;
    If Search.Count < 4 Then
      Exit;
    Version[1][0] := StrToInt(Search[0]);
    Version[1][1] := StrToInt(Search[1]);
    Version[1][2] := StrToInt(Search[2]);
    Version[1][3] := StrToInt(Search[3]);
  Finally
    Search.Free;
  End;
  If (Version[1][0] > Version[0][0]) Or
    ((Version[0][0] = Version[1][0]) And (Version[1][1] > Version[0][1])) Or
    ((Version[0][0] = Version[1][0]) And (Version[1][1] = Version[0][1]) And (Version[1][2] > Version[0][2])) Or
    ((Version[0][0] = Version[1][0]) And (Version[1][1] = Version[0][1]) And (Version[1][2] = Version[0][2]) And (Version[1][3] > Version[0][3])) Then
  Begin
    Synchronize(SyncroMsgNew);
    If DownUpdate Then
      StartUpdate(ActualVer, FileToDown, FileSize, FileMD5, Author, Mail, WebPage, Comment);
  End
  Else
  If WarnNoNew Then
    Synchronize(SyncroMsgNoNew);
End;

Procedure TThreadUpdater.SyncroMsgNew;
Begin
  If (MessageDlg(Format(MsgNew, [NewVersionName]), mtInformation, [mbYes, mbNo], 0) = 6) Then
    DownUpdate := True;
End;

Procedure TThreadUpdater.SyncroMsgNoNew;
Begin
  MessageDlg(MsgNoNew, mtInformation, [mbOK], 0);
End;

Procedure TThreadUpdater.StartUpdate(ActualVer, FileToDown, FileSize, FileMD5, Author, Mail, WebPage, Comment: String);
Var
  iniSettings: TMemIniFile;
  CommentFile: textfile;
Begin
  ExtractOfRes('updater', UpdaterName);
  If (Language <> 'fra') And (Language <> '') Then
    ExtractOfRes('updaterlng' + AnsiLowerCase(Language), ChangeFileExt(UpdaterName, '.' + Language));
  iniSettings := TMemIniFile.Create(ChangeFileExt(UpdaterName, '.ini'));
  Try
    iniSettings.WriteString('Main', 'ActualVer', ActualVer);
    iniSettings.WriteString('Main', 'NewVer', NewVersionName);
    iniSettings.WriteString('Main', 'FileToDown', FileToDown);
    iniSettings.WriteInteger('Main', 'FileSize', StrToInt(FileSize));
    iniSettings.WriteString('Main', 'FileMD5', FileMD5);
    iniSettings.WriteString('Main', 'Author', Author);
    iniSettings.WriteString('Main', 'Mail', Mail);
    iniSettings.WriteString('Main', 'WebPage', WebPage);
    iniSettings.WriteString('Main', 'OldFilePath', Application.ExeName);
    iniSettings.WriteString('Main', 'Language', Language);
    iniSettings.WriteString('Main', 'Skin', Skin);
    iniSettings.WriteBool('Main', 'UpdateAtRun', True);
    iniSettings.UpdateFile;
  Finally
    iniSettings.Free;
  End;
  // ecrire commentaire
  AssignFile(CommentFile, ChangeFileExt(UpdaterName, '.diz'));
  ReWrite(CommentFile);
  WriteLn(CommentFile, Comment);
  CloseFile(CommentFile);
  // lancer updater, quitter
  ShellExecute(Application.Handle, 'open', PChar(UpdaterName), NIL, NIL, SW_SHOWNORMAL);
End;

//pour tlcharger un fichier sur Internet et le retourner sous forme de string
Function TThreadUpdater.DownAndReturnThis(URL: String): String;
Const
  Agent = 'Check for update';
Var
  hFile, HInet: HINTERNET;
  Buffer:       Array[0..32767] Of Char;
  BufRead:      Cardinal;
  BufSize:      Cardinal;
Begin
  // on met le rsultat vide si il y a une erreur
  Result := '';
  HInet  := InternetOpen(PChar(Agent), INTERNET_OPEN_TYPE_PRECONFIG, NIL, NIL, 0);
  If Assigned(HInet) Then
    Try
      //on essaye d'ouvrir l'adresse du fichier
      hFile := InternetOpenUrl(HInet, PChar(URL), NIL, 0, INTERNET_FLAG_RELOAD +
        INTERNET_FLAG_NO_CACHE_WRITE + INTERNET_FLAG_EXISTING_CONNECT +
        INTERNET_FLAG_NO_COOKIES, 0);
      //si on a russi  l'ouvrir
      If Assigned(hFile) Then
        Try
          //on essaie de rcuprer les donnes du fichier
          BufSize := SizeOf(Buffer);
          With TStringStream.Create('') Do
            Try
              While InternetReadFile(hFile, @Buffer, BufSize, BufRead) And (BufRead > 0) Do
                Write(Buffer, BufRead);
              //le rsultat est le fichier tlcharg
              Result := DataString;
            Finally
              Free; //on libre l'lment qui a permet de stocker le tlchargement de fichiers
            End;
        Finally
          InternetCloseHandle(hFile);
        End;
    Finally
      InternetCloseHandle(hinet);
    End;
End;

Initialization
  TUpdater     := NIL;
  StateUpdater := False;

End.

