Unit Main;

Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, WinSkinData, IniFiles, ComCtrls, StdCtrls, ShellApi, MD5Api, WinInet,
  DateUtils, ReInit, MsgResource, ExtCtrls;

Type
  TConf = Record
  Public
    ActualVer:   String[15];
    NewVer:      String[15];
    FileToDown:  String;
    FileSize:    Integer;
    FileSizeStr: String;
    FileMD5:     String;
    Author:      String[50];
    Mail:        String[100];
    WebPage:     String;
    OldFilePath: String;
    Language:    String[3];
    Skin:        String;
    UpdateAtRun: Boolean;
  End;

  TfmMain = Class(TForm)
    SkinData:      TSkinData;
    gbDetails:     TGroupBox;
    gbInfos:       TGroupBox;
    btStartStop:   TButton;
    btQuit:        TButton;
    ProgressBar:   TProgressBar;
    lProgress:     TLabel;
    lPourcent:     TLabel;
    lActualVer:    TLabel;
    lNewVer:       TLabel;
    lAuthor:       TLabel;
    lWebPage:      TLabel;
    lMail:         TLabel;
    lState:        TLabel;
    lSizeDown:     TLabel;
    lTimeLeft:     TLabel;
    reInfoVersion: TRichEdit;
    Label1:        TLabel;
    Timer:         TTimer;
    Procedure FormCreate(Sender: TObject);
    Procedure btQuitClick(Sender: TObject);
    Procedure LabelMouseEnter(Sender: TObject);
    Procedure LabelMouseLeave(Sender: TObject);
    Procedure lWebPageClick(Sender: TObject);
    Procedure btStartStopClick(Sender: TObject);
    Procedure FormDestroy(Sender: TObject);
    Procedure FormCloseQuery(Sender: TObject; Var CanClose: Boolean);
    Procedure lMailClick(Sender: TObject);
    Procedure FormShow(Sender: TObject);
    Procedure TimerTimer(Sender: TObject);
    Procedure FormCanResize(Sender: TObject; Var NewWidth, NewHeight: Integer;
      Var Resize: Boolean);
  Private
    Conf:              TConf;
    PResLanguage:      Longint;
    StopNow:           Boolean;
    LanguageResFile:   String;
    ReadyToClose:      Boolean;
    FStartDate:        TDateTime;
    SizeDown, SpeedDown, Pourcent, ElapsedTime, LimitRefresh, ElapsedTimeMS: Integer;
    sAppName, sNewApp: String;
    Procedure ReadIni;
    Function AffTime(ts: Integer): String;
    Procedure ResetInfos;
    Procedure UpToDateInfos;
    Function DetectionConnexion: Boolean;
    Procedure MakeUpdate;
    Function DownloadFileHttp: Boolean;
    Procedure StartUpdate;
    { Dclarations prives }
  Public
    { Dclarations publiques }
  End;

Var
  fmMain: TfmMain;

Implementation

{$R *.dfm}

Function ApplicationVersionWithBuild(Version: String): String;
Var
  PointBuild: Integer;
Begin
  PointBuild := LastDelimiter('.', Version); //on trouve la position du dernier . Dans le numro
  Result     := Copy(Version, 0, PointBuild - 1) + ' Build: ' + Copy(Version, PointBuild + 1, Length(Version));
End;

Procedure TfmMain.ReadIni;
Var
  iniSettings: TMemIniFile;
Begin
  iniSettings      := TMemIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Conf.ActualVer   := iniSettings.ReadString('Main', 'ActualVer', '');
  Conf.NewVer      := iniSettings.ReadString('Main', 'NewVer', '');
  Conf.FileToDown  := iniSettings.ReadString('Main', 'FileToDown', 'nothing');
  Conf.FileSize    := iniSettings.ReadInteger('Main', 'FileSize', 0);
  Conf.FileMD5     := AnsiUpperCase(iniSettings.ReadString('Main', 'FileMD5', 'XXXXXXX'));
  Conf.Author      := iniSettings.ReadString('Main', 'Author', '');
  Conf.Mail        := iniSettings.ReadString('Main', 'Mail', '');
  Conf.WebPage     := iniSettings.ReadString('Main', 'WebPage', '');
  Conf.OldFilePath := iniSettings.ReadString('Main', 'OldFilePath', 'nothing');
  Conf.Language    := AnsiLowerCase(iniSettings.ReadString('Main', 'Language', 'fra'));
  Conf.Skin        := iniSettings.ReadString('Main', 'Skin', '');
  Conf.UpdateAtRun := iniSettings.ReadBool('Main', 'UpdateAtRun', False);
  iniSettings.Destroy;
End;

Function TfmMain.AffTime(Ts: Integer): String;
Var
  Th: TTimeStamp;
Begin
  If Ts < 0 Then
    Ts := 0;
  Th.Time := ts * 1000;
  Th.Date := 1;
  DateTimeToString(Result, 'hh:nn:ss', TimeStampToDateTime(Th));
End;

Procedure TfmMain.ResetInfos;
Begin
  //  fmPrinc.lSizeDown.Caption := '0 / ' + fmPrinc.Conf.FileSizeStr + ' Ko';
  lTimeLeft.Caption    := MsgZeroTime;
  ProgressBar.Position := 0;
  lPourcent.Caption    := '0 %';
  btStartStop.Caption  := MsgbtStart;
  btStartStop.Enabled  := True;
  btQuit.Enabled       := True;
  //  BorderIcons := [biSystemMenu, biMinimize, biMaximize];
End;

Procedure TfmMain.UpToDateInfos;
Begin
  lTimeLeft.Caption    := MsgZeroTime;
  btStartStop.Enabled  := False;
  lPourcent.Caption    := '100%';
  ProgressBar.Max      := 100;
  ProgressBar.Position := 100;
  btQuit.Enabled       := True;
End;

Function TfmMain.DetectionConnexion: Boolean;
Var
  dwFlags: DWord;
Begin
  dwFlags := INTERNET_CONNECTION_MODEM Or INTERNET_CONNECTION_LAN Or INTERNET_CONNECTION_PROXY;
  Result  := InternetGetConnectedState(@dwFlags, 0);
End;

Procedure TfmMain.MakeUpdate;
Var
  HMutex:  THandle;
  NbError: Integer;
  StopNow: Boolean;
Begin
  StopNow             := False;
  //- desactiver update button, verifier md5 fichier
  btStartStop.Enabled := False;
  lState.Caption      := MsgCheckMD5;
  If AnsiUpperCase(MD5DataToStr(MD5DataFromFile(sNewApp))) <> Conf.FileMD5 Then
  Begin
    lState.Caption := MsgErrMD5;
    ResetInfos;
    DeleteFile(sNewApp);
    Exit;
  End;

  lState.Caption := MsgCheckIfRun;
  Repeat
    SetLastError(NO_ERROR);
    HMutex  := OpenMutex(MUTEX_ALL_ACCESS, False, PChar(MsgAppName));
    NbError := GetLastError;
    If NbError <> ERROR_FILE_NOT_FOUND Then
    Begin
      Application.ProcessMessages;
      Application.BringToFront;
      If Application.MessageBox(PChar(MsgStopBeforeContinue),
        PChar('Mise  jour'),
        MB_OKCANCEL Or MB_ICONWARNING) = idCancel Then
        StopNow := True;
      CloseHandle(HMutex);
    End;
  Until (NbError = ERROR_FILE_NOT_FOUND) Or StopNow;
  If Not StopNow Then
  Begin
    lState.Caption := MsgCopy;
    If BoolToStr(MoveFileEx(PChar(sNewApp), PChar(Conf.OldFilePath), MOVEFILE_COPY_ALLOWED Or MOVEFILE_REPLACE_EXISTING)) = '0' Then
    Begin
      lState.Caption := MsgErrMove;
      ResetInfos;
      DeleteFile(Application.ExeName + '.tmp');
    End
    Else
    Begin
      UpToDateInfos;
      lActualVer.Caption  := Format(MsgActualVer, [Conf.NewVer]);
      lState.Caption      := MsgFinish;
      btStartStop.Caption := MsgRestart;
      btStartStop.Enabled := True;
      MessageDlg(MsgUpdateOk, mtInformation, [mbOK], 0);
    End;
  End
  Else
    ResetInfos;
End;

Function TfmMain.DownloadFileHttp: Boolean;
Const
  BufferSize = 1024;
Var
  hSession, hURL: HInternet;
  Buffer:         Array[1..BufferSize] Of Byte;
  BufferLen:      DWORD;
  FS:             TFileStream;
  FlagsDown:      Int64;
Begin
  Result := False;
  Try
    If Not DetectionConnexion Then
    Begin
      Application.MessageBox(PChar(MsgNoConnection),
        PChar('Erreur'),
        MB_OK Or MB_ICONERROR);
      ResetInfos;
      Exit;
    End;
    LimitRefresh := 0;
    hSession     := InternetOpen(PChar(sAppName), INTERNET_OPEN_TYPE_PRECONFIG, '', '', 0);
    If lState.Caption = MsgStateCancel Then
      FlagsDown := 0
    Else
    Begin
      FStartDate := Now; //Stocker la date
      FlagsDown  := INTERNET_FLAG_RELOAD;
    End;
    lState.Caption := MsgStateDown;
  //  if hSession = nil then
  //  begin
  //    lState0.Caption := 'erreur acces net';
  //    ResetInfos;
  //  end;
    If FileExists(sNewApp) Then
      DeleteFile(sNewApp);
    FS := TFileStream.Create(sNewApp, fmCreate);
    Try
      hURL := InternetOpenURL(hSession, PChar(Conf.FileToDown), NIL, 0, FlagsDown, 0);
      Try
        Repeat
          InternetReadFile(hURL, @Buffer, BufferSize, BufferLen);
          FS.WriteBuffer(Buffer, BufferLen);
          ElapsedTime   := SecondsBetween(Now, FStartDate); //Calculer le temps de tlchargement
          ElapsedTimeMS := MillisecondsBetween(Now, FStartDate);
          If ElapsedTimeMS > LimitRefresh Then
          Begin
            LimitRefresh         := ElapsedTimeMS + 200;
            SizeDown             := fs.Size;
            ProgressBar.Position := SizeDown;
            Try
              Pourcent          := Round(((SizeDown / Conf.FileSize) * 100));
              lPourcent.Caption := IntToStr(Pourcent) + ' %';
            Except
            End;
            Try
              SizeDown       := SizeDown Div 1024;
              SpeedDown      := round(ElapsedTime);
              SpeedDown      := SizeDown Div SpeedDown;
              lState.Caption := Format(MsgStateDown + ' @ %s Ko/sec...', [FormatFloat('0.00', SpeedDown)]);
            Except
            End;
            Try
              SpeedDown         := round(((Conf.FileSize / (fs.Size / ElapsedTime)) - ElapsedTime));
              lTimeLeft.Caption := Format(MsgTime, [AffTime(SpeedDown)]);
            Except
            End;
          End;
          Application.ProcessMessages;
          If StopNow Then
            BufferLen := 0;
        Until BufferLen = 0;
        Result := True;
      Finally
        InternetCloseHandle(hURL)
      End
    Finally
      InternetCloseHandle(hSession);
      FS.Free;
    End;
    If StopNow Then
    Begin
      lState.Caption := MsgStateCancel;
      DeleteFile(sNewApp);
      ResetInfos;
    End
    Else
      MakeUpdate;
  Except
    on E: Exception Do
    Begin
      lState.Caption := MsgUnknowError;
      ResetInfos;
      ShowMessage(E.ClassName + ' erreur, avec le message: ' + E.Message);
    End;
  End;
End;

Procedure TfmMain.StartUpdate;
Begin
  If btStartStop.Caption = MsgbtStart Then
  Begin
    ReadyToClose        := False;
    btStartStop.Caption := MsgbtStop;
    btQuit.Enabled      := False;
    //    BorderIcons := [biSystemMenu, biMinimize, biMaximize];
    StopNow             := False;
    If Not FileExists(sNewApp) Or (AnsiUpperCase(MD5DataToStr(MD5DataFromFile(sNewApp))) <> Conf.FileMD5) Then
      DownloadFileHttp
    Else
      MakeUpdate;
    ReadyToClose := True;
  End
  Else
  If btStartStop.Caption = MsgbtStop Then
  Begin
    StopNow := True;
  End
  Else
  If btStartStop.Caption = MsgRestart Then
  Begin
    ShellExecute(Handle, 'open', PChar(Conf.OldFilePath), NIL, NIL, SW_SHOWNORMAL);
    Close;
  End;
End;

Procedure TfmMain.TimerTimer(Sender: TObject);
Begin
  Timer.Enabled := False;
  reInfoVersion.Perform(WM_VSCROLL, SB_TOP, 0);
  Application.ProcessMessages;
  Application.BringToFront;
  StartUpdate;
End;

Procedure TfmMain.btQuitClick(Sender: TObject);
Begin
  Close;
End;

Procedure TfmMain.btStartStopClick(Sender: TObject);
Begin
  StartUpdate;
End;

Procedure TfmMain.FormCanResize(Sender: TObject; Var NewWidth,
  NewHeight: Integer; Var Resize: Boolean);
Begin
  If NewWidth < 600 Then
    NewWidth := 600;
  If NewHeight < 272 Then
    NewHeight := 272;
End;

Procedure TfmMain.FormCloseQuery(Sender: TObject; Var CanClose: Boolean);
Begin
  CanClose := ReadyToClose;
End;

Procedure TfmMain.FormCreate(Sender: TObject);
Var
  tmpComment: TStringList;
  I:          Integer;
Begin
  ReadyToClose := True;
  ReadIni;
  PResLanguage := 0;
  If (Conf.Language <> 'fra') And (Conf.Language <> '') Then
  Begin
    LanguageResFile := ChangeFileExt(Application.ExeName, '.' + AnsiUpperCase(Conf.Language));
    If FileExists(LanguageResFile) Then
    Begin
      PResLanguage := LoadNewResourceModule(LanguageResFile);
      If PResLanguage <> 0 Then
        ReinitializeForms
      Else
      If PResLanguage = 0 Then
      Begin
          //erreur chargement traduction
        ShowMessage('error translation');
      End;
    End
    Else
      ShowMessage('erreur no file found: ' + LanguageResFile);
  End;
  Conf.FileSizeStr   := IntToStr((Conf.FileSize Div 1024));
  lActualVer.Caption := Format(MsgActualVer, [ApplicationVersionWithBuild(Conf.ActualVer)]);
  lNewVer.Caption    := Format(MsgNewVer, [ApplicationVersionWithBuild(Conf.NewVer)]);
  lSizeDown.Caption  := MsgSize + ' ' + Conf.FileSizeStr + ' Ko';
  lAuthor.Caption    := Format(MsgAuthor, [Conf.Author]);
  lAuthor.Hint       := Conf.Author;
  lMail.Caption      := Conf.Mail;
  lMail.Hint         := Conf.Mail;
  If lMail.Width > 153 Then
  Begin
    lMail.AutoSize := False;
    lMail.Width    := 153;
  End;
  lWebPage.Hint   := Conf.WebPage;
  ProgressBar.Max := Conf.FileSize;
  If FileExists(ChangeFileExt(Application.ExeName, '.diz')) Then
  Begin
    tmpComment := TStringList.Create;
    Try
      //on charge diz dans un strings list
      tmpComment.LoadFromFile(ChangeFileExt(Application.ExeName, '.diz'));
      tmpComment.Text := StringReplace(tmpComment.Text, '\''', '''', [rfReplaceAll]);
      reInfoVersion.Clear;
      //on ecrit les lignes une a une dans le richedit
      For I := 0 To tmpComment.Count - 1 Do
      Begin
        //si pos version au debut d'une ligne on utilise le bleu et gras sinon normal
        If AnsiPos('version ', AnsiLowerCase(tmpComment.Strings[I])) = 1 Then
        Begin
          reInfoVersion.SelAttributes.size  := 12;
          reInfoVersion.SelAttributes.Style := reInfoVersion.SelAttributes.Style + [fsBold];
          reInfoVersion.SelAttributes.color := clBlue;
          reInfoVersion.Lines.Add(tmpComment.Strings[I]);
        End
        Else
        Begin
          reInfoVersion.SelAttributes.Color := clWindowText;
          reInfoVersion.SelAttributes.size  := 10;
          reInfoVersion.SelAttributes.Style := reInfoVersion.SelAttributes.Style - [fsBold];
          reInfoVersion.Lines.Add(tmpComment.Strings[I]);
        End;
      End;
    Finally
      tmpComment.Free;
    End;
  End;
  If (Conf.Skin <> '') And FileExists(Conf.Skin) Then
  Begin
    SkinData.LoadFromFile(Conf.Skin);
    SkinData.Active := True;
    DeleteFile(Conf.Skin);
  End;
  If Conf.ActualVer = Conf.NewVer Then
  Begin
    UpToDateInfos;
    btStartStop.Caption := MsgUpToDate;
    btStartStop.Hint    := MsgUpToDate;
    lState.Caption      := MsgUpToDate;
  End;
  sAppName := ExtractFileName(Application.ExeName);
  sNewApp  := IncludeTrailingPathDelimiter(GetEnvironmentVariable('TEMP')) + ChangeFileExt(ExtractFileName(Application.ExeName), '_tmp.exe');
End;

Procedure TfmMain.FormDestroy(Sender: TObject);
Begin
  If PResLanguage <> 0 Then
  Begin
    FreeLibrary(PResLanguage);
    DeleteFile(LanguageResFile);
  End;
{$IFNDEF DEBUG}
  If FileExists(ChangeFileExt(Application.ExeName, '.ini')) Then
    DeleteFile(ChangeFileExt(Application.ExeName, '.ini'));
  If FileExists(ChangeFileExt(Application.ExeName, '.diz')) Then
    DeleteFile(ChangeFileExt(Application.ExeName, '.diz'));
{$ENDIF}
End;

Procedure TfmMain.FormShow(Sender: TObject);
Begin
  If Conf.UpdateAtRun Then
  Begin
    Conf.UpdateAtRun := False;
    Timer.Enabled    := True;
  End;
End;

Procedure TfmMain.lMailClick(Sender: TObject);
Var
  eMail: String;
Begin
  eMail := 'mailto://' + Conf.Mail;
  ShellExecute(handle, 'open', PChar(eMail), NIL, NIL, SW_SHOWDEFAULT);
End;

Procedure TfmMain.lWebPageClick(Sender: TObject);
Begin
  ShellExecute(handle, 'open', PChar(Conf.WebPage), '', '', SW_SHOWMAXIMIZED);
End;

Procedure TfmMain.LabelMouseEnter(Sender: TObject);
Begin
  TLabel(Sender).Font.Style := TLabel(Sender).Font.Style + [fsUnderline];
End;

Procedure TfmMain.LabelMouseLeave(Sender: TObject);
Begin
  TLabel(Sender).Font.Style := TLabel(Sender).Font.Style - [fsUnderline];
End;

End.

